/* ___________________________________________________________________________

    Module: createValidation.js   Version: v0.1
    Author: Guillermo Peña (guillermo.pena.cardano@gmail.com)
    Last update: 06/08/2013
    
        Create and save a new validation

______________________________________________________________________________*/



// __ Modules _____________________________________________________________

var util 	   = require("util")
var uuid 	   = require("node-uuid")
var valMon     = require("../lib/valMon.js")
var EasyWizard = require("../node_modules/easywizard/lib/easyWizard.js")

// __ Private Functions ___________________________________________________

var prepareEvents = function(id) {
	vm.emitter.once('valmon:' + id + ':insert:done', function() {
		console.log("\n  Register inserted");
		ew.emitter.emit('continue')
	})
	vm.emitter.once('valmon:' + id + ':insert:error', 		  function() { error = true })
	vm.emitter.once('valmon:' + id + ':insert:connect:error', function() { error = true })
	vm.emitter.once('valmon:' + id + ':find:error', 		  function() { error = true })
	vm.emitter.once('valmon:' + id + ':find:connect:error',	  function() { error = true })
}


// __ Main Object _________________________________________________________

var ew = new EasyWizard()
var id = uuid.v4().substring(0, 5)
ew.getObject()._id = id // Random _id

var vmJson = { dbname:"test", server:"localhost", port:27017, collection:"validations", logLevel:1 }
var vm = new valMon(vmJson)

// Events 
var error = false
ew.emitter.once('end', function() { console.log("\nThanks for use insertRegister v1.0\n")})
prepareEvents(id)

ew.emitter.on('paused', function() {

	var o = ew.getObject()
	var currentStep = ew.getCurrentStep()

	// Assigning the only validator query name
	if ( currentStep == "1" && o.type == "query" ) o.name = "exists"

	// Parameters
	if ( currentStep == "1.1.1" ) {
		if ( o.parameters == undefined ) o.parameters = new Object()
		if ( parseInt(o.min) ) o.min = parseInt(o.min)
		o.parameters.min = o.min 
		delete o.min
	}
	if ( currentStep == "1.1.2" ) {
		if ( o.parameters == undefined ) o.parameters = new Object()
		if ( parseInt(o.max) ) o.max = parseInt(o.max)
		o.parameters.max = o.max 
		delete o.max
	}
	if ( currentStep == "2a" ) {
		if ( o.parameters == undefined ) o.parameters = new Object()
		o.parameters.collectionTarget = o.collectionTarget 
		delete o.collectionTarget
	}

	// Converting properties string in array of properties
	if ( currentStep == "3") {
		o.properties = o.properties.replace(new RegExp('[ ]+[,]', 'g'), ",") // Removing blancks
		o.properties = o.properties.replace(new RegExp('[,][ ]+', 'g'), ",")
		o.properties = o.properties.split(",")	// Splitting string properties
	}

	// Saving validation 
	if ( currentStep == "5" ) {
		if ( o.enabled == true) {
			console.log("\n  Saving new validation... ")
			if ( o.type == "logic" || o.type == "query" ) {
				o.db = new Object()
				o.db.name = vmJson.dbname
				o.db.server = vmJson.server
				o.db.port = vmJson.port
			}
			vm.insert(ew.getObject()._id, "validations", o)
			if ( o.type == "logic" ) 
				console.log("  Remember your new validation Id: " + o._id)
		} else { 
			console.log("\n  Your new validation wont be saved...")
			ew.emitter.emit('continue')
		}
	} else {
		ew.emitter.emit('continue')
	}

	// Reseting object to create another validation
	if ( o.again === true ) {
		o = ew.resetObject()
		vm = new valMon(vmJson)
		id = uuid.v4().substring(0, 5)
		o._id = id // New random _id
		prepareEvents(id)
		delete o.again

	}
})


var wizardFile = "./createValidation.wizard"
ew.run(wizardFile)