/* ___________________________________________________________________________

    Module: removeRegister.js   Version: v0.1
    Author: Guillermo Peña (guillermo.pena.cardano@gmail.com)
    Last update: 08/08/2013
    
        Remove register/s in a collection

______________________________________________________________________________*/



// __ Modules _____________________________________________________________

var util       = require("util")
var uuid 	   = require("node-uuid")
var valMon     = require("../lib/valMon.js")
var EasyWizard = require("../node_modules/easywizard/lib/easyWizard.js")


process.on('uncaughtException', function(err) { console.error(err.stack) })

// __ Private Methods _____________________________________________________

var prepareEvents = function() {
	id = uuid.v4().substring(0, 5)
	vm.emitter.once('valmon:' + id + ':remove:done', 		  function() { ew.emitter.emit('nextRegToRemove', true) })
	vm.emitter.once('valmon:' + id + ':remove:validate:nok',  function() { ew.emitter.emit('nextRegToRemove', false) })
	vm.emitter.once('valmon:' + id + ':remove:error', 		  function() { error = true })
	vm.emitter.once('valmon:' + id + ':remove:connect:error', function() { error = true })
	vm.emitter.once('valmon:' + id + ':find:error', 		  function() { error = true })
	vm.emitter.once('valmon:' + id + ':find:connect:error',	  function() { error = true })
}

// __ Main Object _________________________________________________________

var ew = new EasyWizard()
var o  = ew.getObject()
var regsToRemove = []
o.filters = new Object()

var vmJson = { dbname:"test", server:"localhost", port:27017, collection:"validations", logLevel:1}
var vm = new valMon(vmJson)

// Events 
var error = false
ew.emitter.once('end', function() { console.log("\nThanks for use insertRegister v1.0\n")})
prepareEvents()

ew.emitter.on('paused', function() {
	
	var o = ew.getObject()
	var currentStep = ew.getCurrentStep()
	var next = true

	// Adding property
	if ( currentStep == 2 ) {
		if ( parseInt(o.value) ) o.value = parseInt(o.value)
		o.filters[o.property] = o.value
		delete o.property
		delete o.value
	}

	// Finding registers
	if ( currentStep == 3 && ew.getNextStep() == 4 ) {
		next = false
		vm.find(id, o.collection, o.filters, function(registers) {
			console.log("\n  " + registers.length + " register/s found.\n")

			regsToRemove = []
			if ( registers.length > 0 ) {	// If register array is not empty
				registers.forEach(function(register) { 
					console.log(" " + util.inspect(register) + "\n")
					regsToRemove.push(register)
				})

			} else { ew.setNextStep(5) }	// If register array is empty we skip removing step

			ew.emitter.emit('continue')		// Emitting 'continue'
		})
	}
	
	// Removing it
	if ( currentStep == 4 && o.confirm == "yes" ) {
		next = false

		if ( regsToRemove.length > 0 ) {
			var i = -1
			ew.emitter.on('nextRegToRemove', function(success) {
				i += 1
				vm = new valMon(vmJson)
				prepareEvents()

				if ( success != null ) {
					if ( success == true ) { console.log("\n  Register removed") }
					else 				   { console.log("\n  Register aborted") }
				}
				
				if ( i == regsToRemove.length ) ew.emitter.emit('continue')
				else {
					var reg = regsToRemove[i]
					console.log("\n  Removing register...\n  " + util.inspect(reg))
					vm.remove(id, o.collection, reg)
				}
			})
			ew.emitter.emit('nextRegToRemove', null)
		} else {
			console.log("Nothing to remove")
			emitter.emit('continue')
		}	
	}

	// Next step
	if ( next ) ew.emitter.emit('continue')
})

var wizardFile = "./removeRegister.wizard"
ew.run(wizardFile)