/* ___________________________________________________________________________

    Module: mongoDao.js   Version: 0.4
    Author: Guillermo Peña (guillermo.pena.cardano@gmail.com)
    Last update: 08/08/2013
    
        DAO wrapper for MongoDb managment
   ___________________________________________________________________________ */

function mongoDao(json) {

	// __ Modules __________________________________________________________________

	var client 		= require("mongodb").MongoClient
	var util        = require("util")
	var events      = require('events')

	// __ Properties ___________________________________________________________
	
	var dbname   = (json.dbname   == undefined || json.dbname   == null) ? "test" 	   : json.dbname
	var server   = (json.server   == undefined || json.server   == null) ? "localhost" : json.server
	var port     = (json.port     == undefined || json.port     == null) ? 27017       : json.port
	var logLevel = (json.logLevel == undefined || json.logLevel == null) ? 2 		   : json.logLevel	
	//  log Level --> [ 0: Dont log | 1: log only important | 2: log all ]

	var emitter = new events.EventEmitter // Event emitter

	// __ Getters/Setters ______________________________________________________

	var getLogLevel = function() { return logLevel }

	// __ Private Methods ______________________________________________________

	// Connect to database
	var connect = function(args, callback) {
  
		
		var connection = "mongodb://" + server + ":" + port + "/" + dbname
		if ( logLevel > 1 ) {
			console.log('  MongoDao : ' + args.id + ' : connect : Connecting to db ' + connection)
		}

		try { client.connect(connection, callback) }
		catch (e) { console.log('Error: ' + e);}
	}

	// Check and get connect status
	var getConnectStatus = function(err, id) {
  
		if (err) { 
			emitter.emit('mongodao:' + id + ':connect:error', err) 
			return false
		} else { 
			emitter.emit('mongodao:' + id + ':connect:done')
			return true 
		}
	}
	
	// __ Public Methods _______________________________________________________


	// Open database, find registers by filters and make anything with it, returning register list if you want
	var find = function(args, callback) {

		connect(args, function(err, db){
			if ( getConnectStatus(err, args.id) ) { 
				if ( logLevel > 1 ) { 
					console.log('  MongoDao : ' + args.id + " : find : Finding registers into collection '" + args.collection + "' ...") 
					console.log('  MongoDao : ' + args.id + " : find : Filters : " + util.inspect(args.filters)) 
				}

				var collection = db.collection(args.collection)
					.find(args.filters)
					.toArray( function(err, registers) {
						if (err) { emitter.emit('mongodao:' + args.id + ':find:error', err) }
						else {
							if ( logLevel > 1 ) 
								console.log('  MongoDao : ' + args.id + " : find : " + registers.length + ' register/s found')
							db.close()
							emitter.emit('mongodao:' + args.id + ':find:done', registers)
							callback(registers, args) // Make anything with result	
						}

						return registers
				})
			}
		})
	}

	// Insert register in database
	var insert = function(args, callback) {

		connect(args, function(err, db){
			if ( getConnectStatus(err, args.id) ) { 
				if ( logLevel > 1 )
					console.log('  MongoDao : ' + args.id + ' : insert : Inserting register into collection ' + args.collection + ' ...')

				var collection = db.collection(args.collection)
					.insert(args.register, { 'safe':true },  function(err) {
						if (err) { emitter.emit('mongodao:' + args.id + ':insert:error', err) }
						else 	 { emitter.emit('mongodao:' + args.id + ':insert:done') }
						db.close()
					})
			}
		})
	}

	// Remove register in database
	var remove = function(args, callback) {

		connect(args, function(err, db){
			if ( getConnectStatus(err, args.id) ) { 
				if ( logLevel > 1 ) { 
					console.log('  MongoDao : ' + args.id + ' : remove : Removing registers into collection ' + args.collection + ' ...') 
					console.log('  MongoDao : ' + args.id + ' : remove : Filters : ' + util.inspect(args.filters)) 
				}

				var collection = db.collection(args.collection)
					.remove(args.filters, function(err, registers) {
						if (err) { emitter.emit('mongodao:' + args.id + ':remove:error', err) }
						else {
							emitter.emit('mongodao:' + args.id + ':remove:done')
							db.close()
							callback(registers, args) // Make anything with result	
						}
					})	
			}
		})
	}

	// ___ Returning ___________________________________________________________

	return { "emitter"	   : emitter
		   , "getLogLevel" : getLogLevel
		   , "find"		   : find
		   , "insert"	   : insert
		   , "remove"	   : remove
		   }
}

module.exports = mongoDao