/* ___________________________________________________________________________

    Module: valMon.js   Version: 0.4
    Author: Guillermo Peña (guillermo.pena.cardano@gmail.com)
    Last update: 08/08/2013
    
        DAO wrapper for MongoDb managment with validations
   ___________________________________________________________________________ */

function valMon(json) {

	// __ Modules __________________________________________________________________

	var util       = require("util")
	var clone 	   = require("clone")
	var uuid 	   = require("node-uuid")
	var events     = require('events')
	var mongoDao   = require('./mongoDao.js')

	// __ Properties ___________________________________________________________
	
	var valPath  = process.env.HOME + "/code/ValMon/lib/validators/"
	var dao      = new mongoDao(json)
	var logLevel = json.logLevel
	var emitter  = new events.EventEmitter // Event emitter

	// __ Private Methods ______________________________________________________

	// Validate a register
	var validate = function(id, action, collection, register) {

		if (logLevel > 1) console.log('  ValMon : ' + id + ' : validate : Validating register before ' + action)
		
		// Looking for validations
		var filters = { 'parent'    : undefined
					  , 'collection': collection
				      , 'enabled'	: true
				      }
		if (action == 'insert') filters.applyBeforeInsert = true 
		if (action == 'remove') filters.applyBeforeDelete = true
		find(id, 'validations', filters, function(validations){

			// If any applicable validation exists...
			if ( validations != undefined && validations.length > 0 ) {

				// Checking validations
				var i = 0
				var valid = true // By default...
				emitter.on('next', function() {
					var validation = validations[i]
					i += 1
					var vId        = validation._id
					var valName    = validation.name
					var eventName  = valName + ':' + vId + ':'

					// Loading validator
			        try {
			            var validatorFile = valPath + valName + ".js"
			            var validatorType = require(validatorFile)
			            var validator = new validatorType()
			        } catch (e) {
			            var error = new Error("Unknown validation type '" + valName + "'")
			            emitter.emit(eventName + 'error', error)
			        }

			        // Events
					validator.emitter.once(eventName + 'error', function(e) { 
						valid = false
						console.log( "    Validation '" + valName + "' " + vId + " ...... ERROR: " + e )
						emitter.emit('valmon:' + id + ':validate:error', e)
					})

					validator.emitter.on(eventName + 'ok', function() {
						console.log( "    Validation '" + valName + "' " + vId + " ...... OK!")
						if ( validations.length == i && valid === true )  emitter.emit('valmon:' + id + ':validate:ok')
						if ( validations.length == i && valid === false ) emitter.emit('valmon:' + id + ':validate:nok')
						if ( validations.length > i ) emitter.emit('next')
					})

					validator.emitter.once(eventName + 'nok', function() { 
						valid = false
						console.log( "    Validation '" + valName + "' " + vId + " ...... FAIL!")
						if ( validations.length == i ) emitter.emit('valmon:' + id + ':validate:nok')
						if ( validations.length > i )  emitter.emit('next')
					})

			        // Preparing checking arguments
			        var checkArgs = { "validation": validation
			                   		, "register"  : register 
				                    , "extras"    : { 'action' : action
				                    				, 'dao'    : dao 
				                    				} 
				                    }
				    
					validator.check(checkArgs)	// Checking validation
				})

				// First event execution
				emitter.emit('next') 

			// If any applicable validation does not exist...
			} else { 
				if ( logLevel > 1 ) console.log('  ValMon : ' + id + ' : validate : No validations to apply') 
				emitter.emit('valmon:' + id + ':validate:ok')
			}
		})
	}

	// __ Public Methods _______________________________________________________

	// Add register... if it pass every validation
	var insert = function (id, collection, register) {

		// Creating operation id...
		if ( id == null ) id = uuid.v4().substring(0, 5)
		if ( logLevel > 1 )
			console.log('  ValMon : ' + id + " : insert : Inserting register in collection '" + collection + "'")

		// If register doesn't pass validation...
		emitter.once('valmon:' + id + ':validate:nok', function() { 
			console.log('\n  Insertion failed' ) 
			emitter.emit('valmon:' + id + ':insert:validate:nok')
		})

		// If register passes validation...
		emitter.once('valmon:' + id + ':validate:ok', function() {
			
			// Preparing arguments
			var args = { 'id'        : id
					   , 'collection': collection
					   , 'register'	 : register
					   , 'logLevel'  : json.logLevel
					   }

			// Events
			dao.emitter.once('mongodao:' + id + ':connect:error',
							function(err) { 
								if (logLevel > 1) 
									console.log('  MongoDao : ' + id + ' : connect : Error : ' + err) 
								emitter.emit('valmon:' + id + ':insert:connect:error', err)
							})
			
			dao.emitter.once('mongodao:' + id + ':insert:error', 
							function(err) { 
								if (logLevel > 1) 
									console.log('  MongoDao : ' + id + ' : insert : Error : ' + err) 
								emitter.emit('valmon:' + id + ':insert:error', err)
							})

			dao.emitter.once('mongodao:' + id + ':insert:done', 
							  function() { 
							  	if (logLevel > 1) 
							  		console.log('  MongoDao : ' + id + ' : insert : Register inserted') 
							  	emitter.emit('valmon:' + id + ':insert:done')
							  })

			dao.insert(args, function(){})
		})
		validate(id, 'insert', collection, register)
	}

	// Remove register... if it pass every validation
	var remove = function (id, collection, filters) {

		// Creating operation id...
		if ( id == null ) id = uuid.v4().substring(0, 5)
		if (logLevel > 1) console.log('  ValMon : ' + id + " : remove : Removing register/s from collection '" + collection + "'")

		// If register doesn't pass validation...
		emitter.once('valmon:' + id + ':validate:nok', function() { 
			console.log('\n  Deletion failed' ) 
			emitter.emit('valmon:' + id + ':remove:validate:nok')
		})

		// If register passes validation...
		emitter.once('valmon:' + id + ':validate:ok', function() {
			
			// Preparing arguments
			var args = { 'id'        : id
					   , 'collection': collection
					   , 'filters'	 : filters
					   , 'logLevel'  : json.logLevel
					   }

			// Events
			dao.emitter.once('mongodao:' + id + ':connect:error',
							function(err) { 
								if (logLevel > 1) 
									console.log('  MongoDao : ' + id + ' : connect : Error : ' + err) 
								emitter.emit('valmon:' + id + ':remove:connect:error', err)
							})

			dao.emitter.once('mongodao:' + id + ':remove:error', 
							function(err) { 
								if (logLevel > 1) 
									console.log('  MongoDao : ' + id + ' : remove : Error : ' + err) 
								emitter.emit('valmon:' + id + ':remove:error', err)
							})

			dao.emitter.once('mongodao:' + id + ':remove:done', 
							  function() { 
							  	if (logLevel > 1) 
							  		console.log('  ValMon : ' + id + ' : remove : Register removed') 
							  	emitter.emit('valmon:' + id + ':remove:done')
							  })

			dao.remove(args, function(){})
		})
		validate(id, 'remove', collection, filters)
	}

	// Open database, find registers by filters and make anything with it
	var find = function(id, collection, filters, callback) {

		// Creating operation id...
		if ( id == null ) id = uuid.v4().substring(0, 5)

		// Preparing arguments
		var args = { 'id'        : id
				   , 'collection': collection
				   , 'filters'	 : filters
				   , 'logLevel'  : json.logLevel
				   }

		// Events
		dao.emitter.once('mongodao:' + id + ':connect:error',
						function(err) { 
							if (logLevel > 1) 
								console.log('  MongoDao : ' + id + ' : connect : Error : ' + err)
							emitter.emit('valmon:' + id + ':find:connect:error', err) 
						})

		dao.emitter.once('mongodao:' + id + ':find:error', 
						function(err) { 
							if (logLevel > 1) 
								console.log('  MongoDao : ' + id + ' : find : Error : ' + err)
							emitter.emit('valmon:' + id + ':find:error', err)
						})

		dao.emitter.on('mongodao:' + id + ':find:done'
						, function(registers) {
							emitter.emit('valmon:' + id + ':find:done', registers) 
						})

		dao.find(args, callback)
	}

	// ___ Returning ___________________________________________________________

	return { "emitter" : emitter
		   , "insert"  : insert
		   , "remove"  : remove
		   , "find"	   : find
		   }
}

module.exports = valMon