/* ___________________________________________________________________________

    Module: Exists.js   Version: 0.1
    Author: Guillermo Peña (guillermo.pena.cardano@gmail.com)
    Last update: 09/08/2013
    
        Validation to check if register exists in collection.
   ___________________________________________________________________________*/   
  
function Exists() {

    // ___ Modules _________________________________________________________________
    
    var events = require('events')
    var util = require('util')


    // __ Name, type, event and isValid ________________________________________
    
    var name     = "exists"                 // Name of validator
    var type     = "value"                  // Type [value | query | logic]
    var emitter  = new events.EventEmitter  // Event emitter for 'isValid' event
    var timeout  = 2000                     // Default timeout (milliseconds)
    var logLevel = 2    // log Level --> [ 0: Dont log | 1: log only important | 2: log all ]

    var getType  = function() { return type }
    var getName  = function() { return name }

    // __ Private Method _______________________________________________________   

    var getRegisters = function(dao, daoArgs, callback) { 

        var id = daoArgs.id 
        dao.emitter.once('mongodao:' + id + ':connect:error',
                        function(err) { 
                            if (logLevel > 1) 
                                console.log('  MongoDao : ' + id + ' : connect : Error : ' + err)
                            emitter.emit('valmon:' + id + ':find:connect:error', err) 
                            emitter.emit('clearTimeout')
                        })

        dao.emitter.once('mongodao:' + id + ':find:error', 
                        function(err) { 
                            if (logLevel > 1) 
                                console.log('  MongoDao : ' + id + ' : find : Error : ' + err)
                            emitter.emit('valmon:' + id + ':find:error', err)
                            emitter.emit('clearTimeout')
                        })

        dao.emitter.on('mongodao:' + id + ':find:done'
                        , function(registers) {
                            emitter.emit('clearTimeout')
                        })

        dao.find(daoArgs, callback)
    }

    // __ Main Method __________________________________________________________
    
    var check = function(args) {

        if ( args.extras.logLevel >= 0 && args.extras.logLevel < 3 ) logLevel = args.extras.logLevel

        //Validating arguments
        var eventName = getName() + ':' + args.validation._id + ':'
        var argumentError = new Error("  isNumber : Wrong arguments.\n  Args: " + util.inspect(args))
        if ( ( args === null || args === undefined ) || 
             ( args.register === null || args.register === undefined ) ||
             ( args.validation === null || args.validation === undefined ) ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }
            
        var properties = args.validation.properties
        if ( Array.isArray(properties) === false ||
             ( Array.isArray(properties) && properties.length == 0 ) ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }

        var parameters = args.validation.parameters
        if ( parameters === null || parameters === undefined ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }

        var collectionTarget = parameters.collectionTarget
        if ( collectionTarget === null || collectionTarget === undefined ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }

        var dao = args.extras.dao 
        if ( dao == null || dao == undefined ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }

        // Preparing timeout
        var timeoutError = new Error("  isNumber : Timeout(" + timeout + ") Error")
        if (args.extras != undefined &&
            args.extras != null &&
            typeof args.extras.timeout == "number" ) timeout = args.extras.timeout

        var timeoutId = setTimeout( function() { emitter.emit(eventName + "error", timeoutError) } , timeout )
        emitter.once( 'clearTimeout', function(){ clearTimeout(timeoutId) })

        // Displaying validation
        var id = args.validation._id
        var description = args.validation.description
        if ( logLevel > 0 )
            console.log( "\n  " + getName() + " (id:" + id + "): " 
                       + "Properties " + util.inspect(properties) + " : " 
                       + "Collection target '" + collectionTarget + "'"
                       + " : " + description)

        // Preparing dao arguments
        var filters = new Object()
        for(var i = 0; i < properties.length;i++){
            var property = properties[i]
            if ( args.register[property] != undefined ) filters[property] = args.register[property]
        }
        var daoArgs1 = { 'collection': collectionTarget
                       , 'filters'   : filters
                       , 'logLevel'  : logLevel
                       }
        
        getRegisters(dao, daoArgs1, function(items) {
            console.log("    " + items.length + " registers were found in collection '" + collectionTarget + "'")
            if ( items == null || items.length == 0 ) { emitter.emit(eventName + "nok") }
            else { emitter.emit(eventName + "ok") }
        })
    }
     
     
    // __ Returning ... ________________________________________________________
    
    return { "check" : check 
           , "emitter" : emitter 
           , "getType" : getType
           , "getName" : getName
           }
}

module.exports = Exists