/* ___________________________________________________________________________

    Module: IsNumber.js   Version: 0.2
    Author: Guillermo Peña (guillermo.pena.cardano@gmail.com)
    Last update: 08/08/2013
    
        Validation to check if value is a number.
        If value is null, result is false
        If value is undefined, result is true
        If value is a defined not-null simple NOT-NUMERIC value, result is false
        If value is a defined not-null simple NUMERIC value, result is true
        If value is array, we must check each value of array
   ___________________________________________________________________________*/

function IsNumber() {

    // ___ Modules _________________________________________________________________
    
    var events = require('events')
    var util = require('util')


    // __ Name, type, event and isValid ________________________________________
    
    var name     = "isNumber"               // Name of validator
    var type     = "value"                  // Type [value | query | logic]
    var emitter  = new events.EventEmitter  // Event emitter for 'isValid' event
    var timeout  = 2000                     // Default timeout (milliseconds)
    var logLevel = 2    // log Level --> [ 0: Dont log | 1: log only important | 2: log all ]

    var getType  = function() { return type }
    var getName  = function() { return name }

    // __ Private Method _______________________________________________________   

    var checker = function(value, level) {

        var valid = true
        if (Array.isArray(value)) { // If value is array, we check each one
            if ( logLevel > 0 )
                console.log("    Checking array of values : " + util.inspect(value))
            level += 1
            var j = 0
            var old_j
            emitter.on("ok",  function() { j += 1 })
            emitter.on("nok", function() { j += 1; valid = false})
            while ( j < value.length ) {
                if ( old_j != j ) checker(value[j], level) 
            }
            level -= 1
        } else { // If value is a simple value, we check it
            if (value != undefined && typeof value != "number") valid = false
            if ( logLevel > 0 )
                console.log("    Checking value '" +  value + "' (" +  typeof value + "): " + valid)
        }

        if ( level > 0 ) {
            ( valid ) ? emitter.emit("ok") : emitter.emit("nok")
        } else {
            ( valid ) ? emitter.emit("ok-property") : emitter.emit("nok-property")
        }
    }

    // __ Main Method __________________________________________________________
    
    var check = function(args) {

        if ( args.extras.logLevel >= 0 && args.extras.logLevel < 3 ) logLevel = args.extras.logLevel

        //Validating arguments
        var eventName = getName() + ':' + args.validation._id + ':'
        var argumentError = new Error("  isNumber : Wrong arguments.\n  Args: " + util.inspect(args))
        if ( ( args === null || args === undefined ) || 
             ( args.register === null || args.register === undefined ) ||
             ( args.validation === null || args.validation === undefined ) ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }
            
        var properties = args.validation.properties
        if ( Array.isArray(properties) === false ||
             ( Array.isArray(properties) && properties.length == 0 ) ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }

        // Preparing timeout
        var timeoutError = new Error("  isNumber : Timeout(" + timeout + ") Error")
        if (args.extras != undefined &&
            args.extras != null &&
            typeof args.extras.timeout == "number" ) timeout = args.extras.timeout

        var timeoutId = setTimeout( function() { emitter.emit(eventName + "error", timeoutError) } , timeout )
        emitter.once( 'clearTimeout', function(){ clearTimeout(timeoutId) })
        
        // Checking values
        var id = args.validation._id
        var description = args.validation.description
        var properties  = args.validation.properties
        if ( logLevel > 0 )
            console.log( "\n  isNumber (id:" + id + "): " 
                       + "Properties " + util.inspect(properties)
                       + " : " + description)
        var i = 0
        var globalValid = true
        emitter.on("ok-property",  function() { i += 1 })
        emitter.on("nok-property", function() { i += 1; globalValid = false})
        while ( i < properties.length ) {
            var value = args.register[properties[i]]
            checker(value, 0)
        }

        // Clear timeout and send final event
        ( globalValid ) ? emitter.emit(eventName + 'ok') : emitter.emit(eventName + 'nok')
        emitter.emit('clearTimeout')

    }
     
     
    // __ Returning ... ________________________________________________________
    
    return { "check" : check 
           , "emitter" : emitter 
           , "getType" : getType
           , "getName" : getName
           }
}

module.exports = IsNumber