/* ___________________________________________________________________________

    Module: Or.js   Version: 0.1
    Author: Guillermo Peña (guillermo.pena.cardano@gmail.com)
    Last update: 08/08/2013
    
        Validation to check all of another validations.
   ___________________________________________________________________________*/
   
   
function Or() {
    
    // ___ Modules _________________________________________________________________

    var events   = require('events')
    var util     = require('util')
    var clone    = require('clone')

    var path     = process.env.HOME + "/code/ValMon/lib/"
    var valPath  = path + 'validators/'

    
    // __ Name, type, event and log level ______________________________________
    
    var name     = "or"                    // Name of validator
    var type     = "logic"                  // Type [value | query | logic]
    var emitter  = new events.EventEmitter  // Event emitter for 'isValid' event
    var timeout  = 2000                     // Default timeout (milliseconds)
    var logLevel = 2    // log Level --> [ 0: Dont log | 1: log only important | 2: log all ]

    var getType  = function() { return type }
    var getName  = function() { return name }

    // __ Private Methods ______________________________________________________

    var checker = function(args) {

        // Loading validator
        var validatorType = new require(valPath + args.validation.name + ".js")
        var validator     = new validatorType
        var id            = args.validation._id
        var eventName     = validator.getName() + ':' + id + ':'

        // Events
        var eventName = validator.getName() + ':' + id + ':'

        validator.emitter.once(eventName + 'ok',    function(){
            console.log("    Validation '" + validator.getName() 
                       + "' " + id + "...... OK!")
            //console.log("and:·Emitting " + eventName + 'ok') 
            emitter.emit(eventName + 'ok') 
        })

        validator.emitter.once(eventName + 'nok',   function(){ 
            console.log("    Validation '" + validator.getName() 
                       + "' " + id + "...... FAIL!")
            emitter.emit(eventName + 'nok') 
        })

        validator.emitter.once(eventName + 'error', function(err){
            console.log("    Validation '" + validator.getName() 
                       + "' " + id + "...... ERROR!: " + util.inspect(err))
            emitter.emit(eventName + 'error', err) 
        })

        // Checking validator
        validator.check(args) 
    }

    var getSubvalidations = function(dao, daoArgs, callback) { 

        var id = daoArgs.id 
        dao.emitter.once('mongodao:' + id + ':connect:error',
                        function(err) { 
                            if (logLevel > 1) 
                                console.log('  MongoDao : ' + id + ' : connect : Error : ' + err)
                            emitter.emit('valmon:' + id + ':find:connect:error', err) 
                            emitter.emit('clearTimeout')
                        })

        dao.emitter.once('mongodao:' + id + ':find:error', 
                        function(err) { 
                            if (logLevel > 1) 
                                console.log('  MongoDao : ' + id + ' : find : Error : ' + err)
                            emitter.emit('valmon:' + id + ':find:error', err)
                            emitter.emit('clearTimeout')
                        })

        dao.emitter.on('mongodao:' + id + ':find:done'
                        , function(registers) {
                            emitter.emit('valmon:' + id + ':find:done', registers) 
                            emitter.emit('clearTimeout')
                        })

        dao.find(daoArgs, callback)
    }
     

    // __ Main Method __________________________________________________________
    
    var check = function(args) {

        if ( args.extras.logLevel >= 0 && args.extras.logLevel < 3 ) logLevel = args.extras.logLevel

        //Validating arguments
        var argumentError = new Error("  " + getName() + " : Wrong arguments.\n  Args: " + util.inspect(args))
        if ( ( args === null || args === undefined ) ||
             ( args.register === null || args.register === undefined ) ||
             ( args.validation === null || args.validation === undefined ) ) {
            emitter.emit(getName() + ':' + "error", argumentError)
            return
        }

        var id = args.validation._id
        var description = args.validation.description
        var eventName = getName() + ':' + id + ':'

        var collection = args.validation.collection    
        if ( collection === null || collection === undefined ) 
            emitter.emit(eventName + "error", argumentError)

        var db = args.validation.db 
        if ( ( db === null || db === undefined ) ||
             ( db.name === null || db.name === undefined ) ||
             ( db.server === null || db.server === undefined ) ||
             ( db.port === null || db.port === undefined || typeof db.port != "number" ) ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }

        var dao = args.extras.dao 
        if ( dao == null || dao == undefined ) {
            emitter.emit(eventName + "error", argumentError)
            return
        }

        var action = args.extras.action 
        if ( action == undefined )  {
            emitter.emit(eventName + "error", argumentError)
            return
        }

        // Preparing timeout
        var timeoutError = new Error("  " + getName() + " : Timeout(" + timeout + ") Error")
        if (args.extras != undefined &&
            args.extras != null &&
            typeof args.extras.timeout == "number" ) timeout = args.extras.timeout

        var timeoutId = setTimeout( function() { emitter.emit(eventName + "error", timeoutError) } , timeout )
        emitter.once( 'clearTimeout', function(){ clearTimeout(timeoutId) })

        // Preparing dao arguments
        var filters = { 'parent'    : id
                      , 'enabled'   : true
                      }
        if ( action == 'insert' ) filters.applyBeforeInsert = true 
        if ( action == 'remove' ) filters.applyBeforeDelete = true
        var daoArgs = { 'collection': 'validations'
                      , 'filters'   : filters
                      , 'logLevel'  : logLevel
                      }

        // Getting subvalidations
        getSubvalidations(dao, daoArgs, function(subvalidations) {

            if ( logLevel > 0 )
                console.log( "\n  " + getName() + " (id:" + id + "): " 
                           + subvalidations.length + " subvalidations : " + description )

            // Validating number of subvalidations
            if ( subvalidations == null || subvalidations == undefined || subvalidations.length < 2 ) {
                var notEnoughSubvalidationsError = "Validation '" + getName() + "' (id:" + id + ") " 
                                                 + "has not enough subvalidations"
                emitter.emit(eventName + 'error', notEnoughSubvalidationsError)
            }

            if ( subvalidations.length > 1 ) {

                // Checking subvalidator
                var i = 0
                var valid = false // By default...
                emitter.on('next', function() {

                    var subvalidation = subvalidations[i]
                    i += 1
                    var eventName = subvalidation.name + ':' + subvalidation._id + ':'
                    var eventParentName = getName() + ':' + id + ':'

                    // Final events
                    emitter.once(eventName + 'ok', function(){
                        valid = true
                        subvalidations = []     // If any validator fails, clear array to go out of loop while
                        emitter.emit(eventParentName + 'ok')
                    })

                    emitter.on(eventName + 'nok', function(){
                        if ( subvalidations.length == i && valid === false ) emitter.emit(eventParentName + 'nok')
                        if ( subvalidations.length == i && valid === true  ) emitter.emit(eventParentName + 'ok')
                        if ( subvalidations.length > i ) emitter.emit('next')
                    })

                    emitter.once(eventName + 'error', function(err){
                        valid = false
                        subvalidations = []     // If any validator fails with error, clear array to go out of loop while
                        emitter.emit(eventParentName + 'error', err)
                    })

                    // Checking validation
                    var subArgs = { "validation": subvalidation
                                  , "register"  : args.register 
                                  , "extras"    : { 'action' : action
                                                  , 'dao'    : dao 
                                                  } 
                                  }
                    checker(subArgs)
                })

                // First event execution
                emitter.emit('next') 
            }
        })
    }


    // __ Returning ... ________________________________________________________
    
    return { "check" : check 
           , "emitter" : emitter 
           , "getType" : getType
           , "getName" : getName
           }
}

module.exports = Or
